# ============================================================================ #
#' Print methods for objects of class PostHoc
#'
#' @author Rodrigo Labouriau
#' @param x an object of class PostHoc to be printed
#' @param ... parameters to be passed ro print
#' @examples MM <- glm(Y ~ Treatment+0,  data=DeIdentifiedExample)
#' @examples GG <- posthoc(MM)
#' @examples print(GG)
#' @exportClass PostHoc
#' @export
print.PostHoc <- function(x, ...){
  object <- x

  CI <- object$CI

  if(is.null(CI)){
    stop("The confidence intervals were not available.")
  }

  CI <- round(CI , digits = object$digits)

  Out <- data.frame(CI, object$Grouping)
  Low <- object$SignificanceLevel/2*100
  Up  <- 100-object$SignificanceLevel/2*100
  names(Out) <- c("Est.",
                  paste(Low,"%", sep=""),
                  paste(Up,"%", sep=""),
                  "Group"
  )

  if(is.null(row.names(CI))) row.names(CI) <- paste("Level", 1:(dim(CI)[1]))
  Out <-  data.frame(Levels=row.names(CI),
                    ParameterCI=paste( CI[ ,1], "(", CI[ ,2], "-",
                                       CI[ ,3], ")",object$Grouping , sep="")
    )

    return(Out)
  }

# ============================================================================ #

# ============================================================================ #
#' Plot method for objects of class PostHoc
#'
#' @author Rodrigo Labouriau
#' @param x an object of class PostHoc to be printed
#' @param y an object of class PostHoc to be printed
#' @param ... parameters to be passed to plot
#' @examples MM <- glm(Y ~ Treatment+0,  data=DeIdentifiedExample)
#' @examples GG <- posthoc(MM)
#' @examples plot(GG)
#' @export
plot.PostHoc <- function(x, y, ...)
{

  object <- x

  Cliques <- object$Cliques
  plot(object$Gr, mark.groups = Cliques, ...)
}
# ==============================================================================

# ==============================================================================
#' Summary method for objects of class PostHoc
#'
#' @author Rodrigo Labouriau
#' @param object an object of class PostHoc to be printed
#' @param ... parameters to be passed to summary
#' @examples MM <- glm(Y ~ Treatment+0,  data=DeIdentifiedExample)
#' @examples GG <- posthoc(MM)
#' @examples summary(GG)
#' @export
summary.PostHoc <- function(object, ...){

  if (class(object) != "PostHoc"){
    stop("Sorry, but the object should be of class PostHoc")
  }

  if(is.null(object$CI)){
    stop("The confidence intervals were not available.")
  }

  Pvalues <- as.character(round(object$PvaluesMatrix, digits=object$digits))
  Pvalues[is.na(Pvalues)] <- ""
  dim(Pvalues) <- dim(object$PvaluesMatrix)
  Pvalues <- Pvalues[ , -dim(object$PvaluesMatrix)[2] ]
  Out <- data.frame(object$CI, object$Grouping, Pvalues )
  Low <- object$SignificanceLevel/2*100
  Up  <- 100-object$SignificanceLevel/2*100
  names(Out) <- c("Est.",
                  paste(Low,"%", sep=""),
                  paste(Up,"%", sep=""),
                  "Group",
                  row.names(object$CI)[-length(row.names(object$CI))]
  )
  return(Out)
}
# ==============================================================================

# ============================================================================ #
#' Barplot method for objects of class PostHoc
#'
#' @author Rodrigo Labouriau
#' @param height an object of class PostHoc to be printed
#' @param ... parameters to be passed to the barlot function
#' @examples MM <- glm(Y ~ Treatment+0,  data=DeIdentifiedExample)
#' @examples GG <- posthoc(MM)
#' @examples barplot(GG)
#' @import graphics
#' @export
barplot.PostHoc <- function(height, ...){

  object <- height
  if (class(object) != "PostHoc"){
    stop("Sorry, but the object should be of class PostHoc")
  }

  if(is.null(object$CI)){
    stop("The confidence intervals were not available.")
  }

  Groups <- object$Grouping
  CI <- object$CI
  Effects <- CI[ ,1]

  if(is.null(object$ylim)){
    DeltaSpace <- (abs(max(CI - min(CI))) * 0.10) +  object$space2max
    ylim <- c(min(0, min(CI)), max(CI)+DeltaSpace)
  }

  bp <- barplot(Effects, ylim = object$ylim, col = object$col, xpd = FALSE,
                ylab = object$ylab, xlab = object$xlab, main = object$main)

  arrows(x0 = bp, x1 = bp, y0 = CI[ ,2], y1 = CI[ ,3], lwd = 1,
         angle = 90, col = object$labelsCol, code = 3, length = object$cex.ticks)

  text(x = bp, y = CI[ ,3] + object$space2max/2,
       labels = paste(Groups, object$additionalTextGrouping, sep =""),
       pos = 3, col = object$labelsCol, cex = object$cex.grouping)

}
# ============================================================================ #

# ============================================================================ #
# ============================================================================ #
#' Lines method for objects of class PostHoc
#'
#' @author Rodrigo Labouriau
#' @param x an object of class PostHoc to be printed
#' @param ... parameters to be passed to the function lines
#' @examples MM <- glm(Y ~ Treatment+0,  data = DeIdentifiedExample)
#' @examples GG <- posthoc(MM)
#' @examples lines(GG)
#' @export
lines.PostHoc <- function(x, ...)
{
  object <- x
  if (class(object) != "PostHoc"){
    stop("Sorry, but the object should be of class GroupClusterEff")
  }

  Groups <- object$Grouping

  CI <- object$CI
  Effects <- CI[ ,1]

  Nlevels <- length(Effects)
  # bp <- barplot(Effects, plot = FALSE)
  bp <- 1:Nlevels
  # if(rightshift != 0){
  bp <- bp + object$rightshift
  # }

  if(is.null(object$ylim)){
    DeltaSpace <- (abs(max(CI - min(CI))) * 0.10) +  object$space2max
    ylim <- c(min(0, min(CI)), max(CI)+DeltaSpace)
  }

  if(is.null(object$ylim)){
    ylim <- c(min(object$CI), max(object$CI))
  } else{
    ylim <- object$ylim
  }

  if(object$superpose){ ################
    points(bp, Effects, type = "l", lwd = 3, lty = object$lty,
           col = object$labelsCol)
  } else{
    plot(bp, Effects, ylim = ylim, type = "l", lwd = 3, axes = FALSE,
         ylab = object$ylab, xlab = object$xlab, main = object$main,
         lty = object$lty,
         col = object$labelsCol, ylog = object$ylog)
  }

    if(object$drawAxis){
      Axis(side = 1, at = bp, labels = names(Effects),
                               cex.lab = object$cex.lab)
      Axis(side = 2, labels = TRUE)
    }

    arrows(x0 = bp, x1 = bp, y0 = CI[ ,2], y1 = CI[ ,3], lwd = 1,
           angle = 90, col = object$labelsCol, code = 3, length = object$cex.ticks)

    text(x = bp, y = CI[ ,3] + object$space2max/2,
         labels = paste(Groups, object$additionalTextGrouping, sep =""),
         pos = 3, col = object$labelsColcol, cex = object$cex.grouping)

}
# ============================================================================ #
